/* ###--------------------------------------------------------------### */
/*									*/
/* file		: bvl_drive.h						*/
/* date		: may 31 1994						*/
/* version	: v109							*/
/* author	: VUONG H.N.						*/
/* content	: declaration of functions and global variables used by	*/
/*		  the behavioural decompiler				*/
/*									*/
/* ###--------------------------------------------------------------### */


extern char *bvl_vhdlname();
extern char *bvl_abl2str();
extern char *bvl_printabl();
extern void  bvl_message();
extern char *bvl_vectorize ();
extern void *bvl_vectnam ();
